package ruin;

import java.util.Scanner;

/**
 * This is our first simulation.
 *
 * <a href="http://mathworld.wolfram.com/GamblersRuin.html">See here</a>
 * to learn more about the math.
 *
 * @author Leon Tabak
 * @version 11 February 2015
 */
public class Ruin {

    public static boolean coinLandedHeadsUp() {
        if (Math.random() < 0.5) {
            return true;
        } // if
        else {
            return false;
        } // else
    } // coinLandedHeadsUp()

    public static Outcome visitToACasino(int stake, int goal) {
        int numberOfBets = 0;
        int balance = stake;
        while (balance > 0 && balance < goal) {
            boolean win;
            if (coinLandedHeadsUp()) {
                win = true;
            } // if
            else {
                win = false;
            } // else

            if (win == true) {
                balance = balance + 1;
            } // if
            else {
                balance = balance - 1;
            } // else

            numberOfBets = numberOfBets + 1;
        } // while
        
        Outcome outcome = new Outcome();
        outcome.setWin(balance == goal);
        outcome.setNumberOfBets(numberOfBets);
        outcome.setBalance(balance);
        return outcome;
    } // visitToACasino( int, int )

    /**
     * All of my code is in one method.
     *
     * @param args are the command line arguments---we do not use these.
     */
    public static void main(String[] args) {
        Scanner scanner = new Scanner(System.in);

        System.out.print("Enter the amount of money that you "
                + "will take to the casino: ");
        int stake = scanner.nextInt();

        System.out.print("How much money do you want to win "
                + "before you leave the casino: ");
        int goal = scanner.nextInt();

        System.out.print("How many times do you want to run the experiment: ");
        int numberOfTrials = scanner.nextInt();

        int numberOfSuccesses = 0;
        int numberOfBetsOnSuccessfulTrips = 0;
        
        int numberOfFailures = 0;
        int numberOfBetsOnUnsuccessfulTrips = 0;
        
        for (int i = 0; i < numberOfTrials; i++) {
            Outcome result = visitToACasino(stake, goal);
            
            if( result.isWin() ) {
                numberOfSuccesses++;
                numberOfBetsOnSuccessfulTrips += result.getNumberOfBets();
            } // if
            else {
                numberOfFailures++;
                numberOfBetsOnUnsuccessfulTrips += result.getNumberOfBets();
            } // else
            
            System.out.println( result.toString() );
//            System.out.println("number of bets = " + numberOfBets);
//            System.out.println("balance = " + balance);
        } // for

        double meanSuccess = ((double) numberOfBetsOnSuccessfulTrips)/numberOfSuccesses;
        double meanFailure = ((double) numberOfBetsOnUnsuccessfulTrips)/numberOfFailures;
        
        System.out.println( "mean number of bets needed to win: " + meanSuccess );
        System.out.println( "mean number of bets needed to bankrkupt: " + meanFailure );
        
        
        
    } // main( String [] )

} // Ruin
